/*
* this is a simple menu using the current LC / Encoder code 
*/
 int16_t currentSelectedMenu=0;
 volatile int16_t nextSelectedMenu = currentSelectedMenu;
char valuePrefix = ' ';
menuStruct menu[] = {
  {'p',0,maxPrograms-1,1,&currentProgram,&nextProgram},
  {'d',1,200,1,&currentDelay,&nextDelay},
  {'c',0,numberOfPalettes-1,1,&currentPalette,&nextPalette},
  {'b',0,255,1,&currentBrightness,&nextBrightness}
};
uint8_t menuEntries = sizeof(menu)/sizeof(menuStruct);

void genericChangeHandler(int16_t newValue)
{
  #if DEBUG_MENU
  Serial << "GCH "<< valuePrefix << "("<<newValue<<")"<<endl;
  Serial << (long)encoderPos << endl;
  #endif
  printNumber(newValue);
  lc.setChar(0,3,valuePrefix,false);
}

void menuChangeHandler(int16_t newValue)
{
  #if DEBUG_MENU
  Serial << "Menu "<<newValue<<' '<<currentSelectedMenu;
  #endif
  currentSelectedMenu = newValue;
  #if DEBUG_MENU
  Serial << " "<<currentSelectedMenu<<endl;
  #endif
  displayMenu();
  #if DEBUG_MENU
  Serial << "endMenu"<<endl;
  #endif
}


void displayMenu()
{
  lc.clearDisplay(0);
  int lcdPos = 3;
  for(uint8_t menuIndex=0;menuIndex<menuEntries;menuIndex++){
    lc.setChar(0,lcdPos--,menu[menuIndex].code, currentSelectedMenu == menuIndex);
  }
}


void selectMenuState()
{
  #if DEBUG_MENU
  Serial <<"selectMenuState"<<endl;
  #endif
  currentState = stateMenu;
  nextSelectedMenu=currentSelectedMenu = 0;
  encoderMinValue = 0;
  encoderMaxValue = menuEntries -1;
  encoderPos = &nextSelectedMenu;
  *encoderPos = currentSelectedMenu;
  encoderStep = 1;
  displayMenu();
  encoderValueChangeCallback = menuChangeHandler;
}
/** central menu input handling state machine */
void clickHandler(uint8_t clicks, boolean wasLongClick)
{
  #if DEBUG_MENU
  Serial << "Got a click "<<clicks<<" "<<wasLongClick<<endl;
  #endif
  if(wasLongClick){                              // cancel all value changes
    currentState = stateCancel;
  }
  switch(currentState){
    case stateNone:
      if(clicks == 1){
        if(!wasLongClick){ // we activate the menu
          #if DEBUG_MENU
          Serial << "None -> Menu"<<endl;
          #endif
          selectMenuState();
        }
      }
    break;
    case stateMenu:
    if(clicks == 1){
      if( (currentSelectedMenu >=0) && (currentSelectedMenu<menuEntries)){
        #if DEBUG_MENU
        Serial << "Menu entry "<<currentSelectedMenu<<" "<<menu[currentSelectedMenu].code<<" selected";
        #endif
        // we got a selected menu entry
        currentState = stateValueChange;
        encoderMaxValue = menu[currentSelectedMenu].max;
        encoderMinValue = menu[currentSelectedMenu].min;
        encoderStep = menu[currentSelectedMenu].step;
        encoderPos = menu[currentSelectedMenu].tempValue;
        *encoderPos = *menu[currentSelectedMenu].value;
        encoderValueChangeCallback = genericChangeHandler;
        valuePrefix = menu[currentSelectedMenu].code;
        genericChangeHandler(*menu[currentSelectedMenu].value);
      }else{
        #if DEBUG_MENU
        Serial << "Illegal menu index: "<<currentSelectedMenu<<endl;
        #endif
      }
    }
    break;
    case stateValueChange:
    {
      selectMenuState();
    }
    break;
    case stateCancel:
    default:
    {
      lc.clearDisplay(0);
      encoderValueChangeCallback = NULL;
      currentSelectedMenu = -1;
      currentState = stateNone;
      encoderPos = NULL;
    }
    break;
  }
}


