/******************************************************
Collection of helper methos for manipulating Pixels
*******************************************************/

/* returns the led position of the pixel in the given band */
uint8_t LEDForPosInBand(uint8_t band, uint8_t pixel)
{
  return pixelForPosInBand(band,pixel) + bands[band].pixelOffset;
}
/* returns the pixel position inside a band*/
uint8_t pixelForPosInBand(uint8_t band, uint8_t pixel)
{
  uint8_t ledCount = bands[band].pixelsInBand;
  uint8_t ledStart = bands[band].pixelOffset;
  while(pixel > ledCount-1){
    pixel = pixel - ledCount;
  }
  return pixel;
}


void setPixelInBand(uint8_t band, uint8_t pixel,CRGB color)
{
  pixel=pixel % bands[band].pixelsInBand;
  uint16_t p=pixel+bands[band].pixelOffset;
  leds[p]=color;
}

void fillBand(uint8_t band,CRGB color)
{
  uint16_t pixelOffset = bands[band].pixelOffset;
  uint8_t pixelsInBand = bands[band].pixelsInBand;
  for(uint16_t pixel = pixelOffset; pixel < pixelOffset + pixelsInBand-1; pixel ++){
    leds[pixel] = color;
  }
}


void runPixelInBand(uint8_t band, CRGB background, CRGB foreground,uint8_t pixelPos,uint8_t pixelCount,fract8 fIn, fract8 fOut,boolean ccw)
{
  uint8_t ledCount = bands[band].pixelsInBand;
  uint8_t ledStart = bands[band].pixelOffset;
  uint8_t pp = pixelPos ;
  uint8_t p2 = pixelPos+1 *-1*ccw;
  if(pp>ledCount-1){
    pp=pixelPos-ledCount ;
  }
  if(p2>ledCount-1){
    p2=pixelPos-ledCount+1*-1*ccw;
  }
  uint8_t pixel = LEDForPosInBand(band,pixelPos);
  uint8_t pixel1 = LEDForPosInBand(band,pixelPos+1*-1*ccw);

  #if DEBUG_EFFECTS
  if(pp!=pixelPos){
    Serial <<"pp: "<<pp<<", Pixel: "<<pixelPos<<endl;
  }else{
    Serial <<"Pixel: "<<pixelPos<<" ("<<leds[pixel].getLuma()<<")"<<endl;
  }
  #endif
  // blend to background color except of pixel
  for(uint8_t t=ledStart;t<ledStart+ledCount;t++){
    if((t!=pixel) || (t!= pixel1)){
      CRGB start = leds[t];
      leds[t]=blend(start,background,fOut);
   }
  }
  leds[pixel] = blend(leds[pixel],foreground,fIn);
  leds[pixel1] = blend(leds[pixel1],leds[pixel],fIn);
}
